@@ -93,15 +93,24 @@ class JGenerators(Enum):
     WapoGenerator = autoclass('io.anserini.index.generator.WashingtonPostGenerator')
 
 # Query Generators
-
 JQueryGenerator = autoclass('io.anserini.search.query.QueryGenerator')
 JBagOfWordsQueryGenerator = autoclass('io.anserini.search.query.BagOfWordsQueryGenerator')
 JCovid19QueryGenerator = autoclass('io.anserini.search.query.Covid19QueryGenerator')
 
+# Query building
+JQueryGeneratorUtils = autoclass('io.anserini.search.query.QueryGeneratorUtils')
+
+JTerm = autoclass('org.apache.lucene.index.Term')
+JBooleanClause = autoclass('org.apache.lucene.search.BooleanClause')
+JBoostQuery = autoclass('org.apache.lucene.search.BoostQuery')
+JTermQuery = autoclass('org.apache.lucene.search.TermQuery')
+JQuery = autoclass('org.apache.lucene.search.Query')
+
 ### Collection
 
 class JCollections(Enum):
     CarCollection = autoclass('io.anserini.collection.CarCollection')
+    Cord19AbstractCollection = autoclass('io.anserini.collection.Cord19AbstractCollection')
     ClueWeb09Collection = autoclass('io.anserini.collection.ClueWeb09Collection')
     ClueWeb12Collection = autoclass('io.anserini.collection.ClueWeb12Collection')
     HtmlCollection = autoclass('io.anserini.collection.HtmlCollection')
@@ -112,4 +121,3 @@ class JCollections(Enum):
     TweetCollection = autoclass('io.anserini.collection.TweetCollection')
     WashingtonPostCollection = autoclass('io.anserini.collection.WashingtonPostCollection')
     WikipediaCollection = autoclass('io.anserini.collection.WikipediaCollection')
-
