@@ -24,6 +24,9 @@ from urllib.request import urlretrieve
 from pyserini.analysis import pyanalysis
 from pyserini.index import pyutils
 from pyserini.pyclass import JString
+from pyserini.search import pysearch
+from pyserini.vectorizer import TfidfVectorizer
+from sklearn.naive_bayes import MultinomialNB
 
 
 class TestIndexUtils(unittest.TestCase):
@@ -40,7 +43,24 @@ class TestIndexUtils(unittest.TestCase):
         tarball.extractall(self.index_dir)
         tarball.close()
 
-        self.index_utils = pyutils.IndexReaderUtils('{}lucene-index.cacm'.format(self.index_dir))
+        self.index_path = os.path.join(self.index_dir, 'lucene-index.cacm')
+        self.searcher = pysearch.SimpleSearcher(self.index_path)
+        self.index_utils = pyutils.IndexReaderUtils(self.index_path)
+
+    def test_tfidf_vectorizer(self):
+        vectorizer = TfidfVectorizer(self.index_path, min_df=5)
+        train_docs = ['CACM-0239', 'CACM-0440', 'CACM-3168', 'CACM-3169']
+        train_labels = [1, 1, 0, 0]
+        test_docs = ['CACM-0634', 'CACM-3134']
+        train_vectors = vectorizer.get_vectors(train_docs)
+        test_vectors = vectorizer.get_vectors(test_docs)
+        clf = MultinomialNB()
+        clf.fit(train_vectors, train_labels)
+        pred = clf.predict_proba(test_vectors)
+        self.assertAlmostEqual(0.49975694, pred[0][0], places=8)
+        self.assertAlmostEqual(0.50024306, pred[0][1], places=8)
+        self.assertAlmostEqual(0.51837413, pred[1][0], places=8)
+        self.assertAlmostEqual(0.48162587, pred[1][1], places=8)
 
     def test_terms_count(self):
         # We're going to iterate through the index and make sure we have the correct number of terms.
@@ -80,13 +100,12 @@ class TestIndexUtils(unittest.TestCase):
         self.assertEqual(df, 74)
         self.assertEqual(cf, None)
 
-        analyzer = pyanalysis.get_lucene_analyzer(stemming=False, stopwords=False)
-        df_no_stem, cf_no_stem = self.index_utils.get_term_counts('retrieval', analyzer)
+        df_no_stem, cf_no_stem = self.index_utils.get_term_counts('retrieval', analyzer=None)
         # 'retrieval' does not occur as a stemmed word, only 'retriev' does.
         self.assertEqual(df_no_stem, 0)
         self.assertEqual(cf_no_stem, 0)
 
-        df_no_stopword, cf_no_stopword = self.index_utils.get_term_counts('on', analyzer)
+        df_no_stopword, cf_no_stopword = self.index_utils.get_term_counts('on', analyzer=None)
         self.assertEqual(df_no_stopword, 326)
         self.assertEqual(cf_no_stopword, 443)
 
@@ -108,22 +127,22 @@ class TestIndexUtils(unittest.TestCase):
     def test_postings2(self):
         self.assertIsNone(self.index_utils.get_postings_list('asdf'))
 
-        postings = list(self.index_utils.get_postings_list('retrieval', analyze=True))
+        postings = list(self.index_utils.get_postings_list('retrieval'))
         self.assertEqual(len(postings), 138)
 
         # If we don't analyze, then we can't find the postings list:
-        self.assertIsNone(self.index_utils.get_postings_list('retrieval', analyze=False))
+        self.assertIsNone(self.index_utils.get_postings_list('retrieval', analyzer=None))
 
         # Supply the analyzed form directly, and we're good:
-        postings = list(self.index_utils.get_postings_list('retriev', analyze=False))
+        postings = list(self.index_utils.get_postings_list('retriev', analyzer=None))
         self.assertEqual(len(postings), 138)
-        postings = list(self.index_utils.get_postings_list(self.index_utils.analyze('retrieval')[0], analyze=False))
+        postings = list(self.index_utils.get_postings_list(self.index_utils.analyze('retrieval')[0], analyzer=None))
         self.assertEqual(len(postings), 138)
 
         # Test utf encoding:
         self.assertEqual(self.index_utils.get_postings_list('zoölogy'), None)
-        self.assertEqual(self.index_utils.get_postings_list('zoölogy', analyze=False), None)
-        self.assertEqual(self.index_utils.get_postings_list('zoölogy', analyze=True), None)
+        self.assertEqual(self.index_utils.get_postings_list('zoölogy', analyzer=None), None)
+        self.assertEqual(self.index_utils.get_postings_list('zoölogy'), None)
 
     def test_doc_vector(self):
         doc_vector = self.index_utils.get_document_vector('CACM-3134')
@@ -131,6 +150,9 @@ class TestIndexUtils(unittest.TestCase):
         self.assertEqual(doc_vector['inform'], 8)
         self.assertEqual(doc_vector['retriev'], 7)
 
+    def test_doc_vector_invalid(self):
+        self.assertTrue(self.index_utils.get_document_vector('foo') is None)
+
     def test_doc_vector_matches_index(self):
         # From the document vector, look up the term frequency of "information".
         doc_vector = self.index_utils.get_document_vector('CACM-3134')
@@ -146,6 +168,12 @@ class TestIndexUtils(unittest.TestCase):
                 # The tf values should match.
                 self.assertEqual(postings_list[i].tf, 8)
 
+    def test_doc_invalid(self):
+        self.assertTrue(self.index_utils.doc('foo') is None)
+        self.assertTrue(self.index_utils.doc_contents('foo') is None)
+        self.assertTrue(self.index_utils.doc_raw('foo') is None)
+        self.assertTrue(self.index_utils.doc_by_field('foo', 'bar') is None)
+
     def test_doc_raw(self):
         raw = self.index_utils.doc('CACM-3134').raw()
         self.assertTrue(isinstance(raw, str))
@@ -181,8 +209,35 @@ class TestIndexUtils(unittest.TestCase):
                          self.index_utils.doc_by_field('id', 'CACM-3134').docid())
 
     def test_bm25_weight(self):
-        self.assertAlmostEqual(self.index_utils.compute_bm25_term_weight('CACM-3134', 'inform'), 1.925014, places=5)
-        self.assertAlmostEqual(self.index_utils.compute_bm25_term_weight('CACM-3134', 'retriev'), 2.496352, places=5)
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'inform', analyzer=None, k1=1.2, b=0.75),
+            1.925014, places=5)
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'information', k1=1.2, b=0.75),
+            1.925014, places=5)
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'retriev', analyzer=None, k1=1.2, b=0.75),
+            2.496352, places=5)
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'retrieval', k1=1.2, b=0.75),
+            2.496352, places=5)
+
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'inform',  analyzer=None),
+            2.06514, places=5)
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'information'),
+            2.06514, places=5)
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'retriev',  analyzer=None),
+            2.70038, places=5)
+        self.assertAlmostEqual(
+            self.index_utils.compute_bm25_term_weight('CACM-3134', 'retrieval'),
+            2.70038, places=5)
+
+        self.assertAlmostEqual(self.index_utils.compute_bm25_term_weight('CACM-3134', 'fox',  analyzer=None),
+                               0., places=5)
+        self.assertAlmostEqual(self.index_utils.compute_bm25_term_weight('CACM-3134', 'fox'), 0., places=5)
 
     def test_docid_converstion(self):
         self.assertEqual(self.index_utils.convert_internal_docid_to_collection_docid(1), 'CACM-0002')
@@ -196,6 +251,50 @@ class TestIndexUtils(unittest.TestCase):
             # Should fail when pyjnius has solved this internally.
             JString('zoölogy')
 
+    def test_query_doc_score_default(self):
+        queries = ['information retrieval', 'databases']
+
+        for query in queries:
+            hits = self.searcher.search(query)
+
+            # We're going to verify that the score of each hit is about the same as the output of
+            # compute_query_document_score
+            for i in range(0, len(hits)):
+                self.assertAlmostEqual(hits[i].score,
+                                       self.index_utils.compute_query_document_score(hits[i].docid, query), places=4)
+
+    def test_query_doc_score_custom_similarity(self):
+        custom_bm25 = pysearch.LuceneSimilarities.bm25(0.8, 0.2)
+        queries = ['information retrieval', 'databases']
+        self.searcher.set_bm25(0.8, 0.2)
+
+        for query in queries:
+            hits = self.searcher.search(query)
+
+            # We're going to verify that the score of each hit is about the same as the output of
+            # compute_query_document_score
+            for i in range(0, len(hits)):
+                self.assertAlmostEqual(hits[i].score,
+                                       self.index_utils.compute_query_document_score(
+                                           hits[i].docid, query, similarity=custom_bm25), places=4)
+
+        custom_qld = pysearch.LuceneSimilarities.qld(500)
+        self.searcher.set_qld(500)
+
+        for query in queries:
+            hits = self.searcher.search(query)
+
+            # We're going to verify that the score of each hit is about the same as the output of
+            # compute_query_document_score
+            for i in range(0, len(hits)):
+                self.assertAlmostEqual(hits[i].score,
+                                       self.index_utils.compute_query_document_score(
+                                           hits[i].docid, query, similarity=custom_qld), places=4)
+
+    def test_index_stats(self):
+        self.assertEqual(3204, self.index_utils.stats()['documents'])
+        self.assertEqual(14363, self.index_utils.stats()['unique_terms'])
+
     def tearDown(self):
         os.remove(self.tarball_name)
         shutil.rmtree(self.index_dir)
